/**
 * @copyright   2017-2018, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { Struct } from 'optimal';
import Context from './Context';
import { RoutineInterface } from './Routine';
import { TaskInterface } from './Task';
import { ToolInterface } from './Tool';
import { Debugger } from './types';
export interface AggregatedResponse {
    errors: Error[];
    results: any[];
}
export default class Executor<To extends Struct = {}> {
    context: Context;
    debug: Debugger;
    options: To;
    tool: ToolInterface;
    constructor(tool: ToolInterface, context: Context, options?: Partial<To>);
    /**
     * Aggregate and partition errors and results into separate collections.
     */
    aggregateResponse(responses: any[]): AggregatedResponse;
    /**
     * Execute either a task or routine.
     */
    execute<T>(task: TaskInterface | RoutineInterface, value?: T, wasParallel?: boolean): Promise<any>;
    /**
     * Execute a routine with the provided value.
     */
    executeRoutine<T>(routine: RoutineInterface, value?: T, wasParallel?: boolean): Promise<any>;
    /**
     * Execute a task with the provided value.
     */
    executeTask<T>(task: TaskInterface, value?: T, wasParallel?: boolean): Promise<any>;
    /**
     * Importing Routine causes a circular reference, so we can't use an instanceof check,
     * so we need to hackily check this another way.
     * */
    getInstanceType(task: TaskInterface | RoutineInterface): string;
    /**
     * Method to execute tasks. Must be defined in sub-classes.
     */
    run<T>(tasks: TaskInterface[], value?: T): void;
}
