/**
 * @copyright   2017-2018, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { Struct } from 'optimal';
import { ConsoleInterface } from './Console';
import Module, { ModuleInterface } from './Module';
import { TaskInterface } from './Task';
import { ColorType, ColorPalette } from './types';
export declare const SLOW_THRESHOLD = 10000;
export interface ReporterInterface<T = any> extends ModuleInterface {
    console: ConsoleInterface;
    lines: T[];
    bootstrap(): void;
}
export default class Reporter<T, To extends Struct = {}> extends Module<To> implements ReporterInterface {
    console: ConsoleInterface;
    lines: T[];
    startTime: number;
    stopTime: number;
    /**
     * Register console listeners.
     */
    bootstrap(): void;
    /**
     * Add a line to be rendered.
     */
    addLine(line: T): this;
    /**
     * Display an error and it's stack.
     */
    displayError(error: Error): void;
    /**
     * Find a line using a callback
     */
    findLine(callback: (item: T) => boolean): T | undefined;
    /**
     * Return specific colors based on chosen theme.
     */
    getColorPalette(): ColorPalette;
    /**
     * Return a specific color for each task status.
     */
    getColorType(task: TaskInterface): ColorType;
    /**
     * Calculate the elapsed time and highlight as red if over the threshold.
     */
    getElapsedTime(start: number, stop?: number, highlight?: boolean): string;
    /**
     * Set start time.
     */
    handleBaseStart: () => void;
    /**
     * Set stop time and render.
     */
    handleBaseStop: () => void;
    /**
     * Create an indentation based on the defined length.
     */
    indent(length?: number): string;
    /**
     * Remove a line to be rendered.
     */
    removeLine(callback: (item: T) => boolean): this;
    /**
     * Create a chalk formatted string with accessible colors and modifiers applied.
     */
    style(message: string, type?: ColorType, modifiers?: ('reset' | 'bold' | 'dim' | 'italic' | 'underline' | 'inverse' | 'hidden')[]): string;
}
