/**
 * @copyright   2017-2018, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { Struct } from 'optimal';
import Context from './Context';
import { Status } from './types';
export interface TaskInterface {
    startTime: number;
    status: Status;
    statusText: string;
    stopTime: number;
    tasks: TaskInterface[];
    title: string;
    isPending(): boolean;
    isRunning(): boolean;
    isSkipped(): boolean;
    hasFailed(): boolean;
    hasPassed(): boolean;
    run<T>(context: Context, initialValue?: T | null): Promise<any>;
    skip(condition?: boolean): this;
}
export declare type TaskAction<Tx extends Context> = (context: Tx, value: any, task: TaskInterface) => any | Promise<any>;
export default class Task<To extends Struct, Tx extends Context> implements TaskInterface {
    action: TaskAction<Tx> | null;
    context: Tx;
    options: To;
    title: string;
    startTime: number;
    status: Status;
    statusText: string;
    stopTime: number;
    tasks: TaskInterface[];
    constructor(title: string, action?: TaskAction<Tx> | null, options?: Partial<To>);
    /**
     * Return true if the task failed when executing.
     */
    hasFailed(): boolean;
    /**
     * Return true if the task executed successfully.
     */
    hasPassed(): boolean;
    /**
     * Return true if the task has not been executed yet.
     */
    isPending(): boolean;
    /**
     * Return true if the task is currently running.
     */
    isRunning(): boolean;
    /**
     * Return true if the task was or will be skipped.
     */
    isSkipped(): boolean;
    /**
     * Run the current task by executing it and performing any before and after processes.
     */
    run<T>(context: Tx, initialValue?: T): Promise<any>;
    /**
     * Set the context to be passed around.
     */
    setContext(context: Tx): this;
    /**
     * Mark a task as skipped if the condition is true.
     */
    skip(condition?: boolean): this;
}
